@@ -12,10 +12,10 @@ extern crate log;
 #[macro_use]
 extern crate rustc_middle;
 
+mod chalk;
 mod dropck_outlives;
 mod evaluate_obligation;
 mod implied_outlives_bounds;
-pub mod lowering;
 mod normalize_erasing_regions;
 mod normalize_projection_ty;
 mod type_op;
@@ -26,7 +26,7 @@ pub fn provide(p: &mut Providers<'_>) {
     dropck_outlives::provide(p);
     evaluate_obligation::provide(p);
     implied_outlives_bounds::provide(p);
-    lowering::provide(p);
+    chalk::provide(p);
     normalize_projection_ty::provide(p);
     normalize_erasing_regions::provide(p);
     type_op::provide(p);
