@@ -1,14 +1,15 @@
 use crate::utils::{
-    is_normalizable, last_path_segment, match_def_path, paths, snippet, span_lint, span_lint_and_then, sugg,
+    is_normalizable, last_path_segment, match_def_path, paths, snippet, span_lint, span_lint_and_sugg,
+    span_lint_and_then, sugg,
 };
 use if_chain::if_chain;
-use rustc::ty::{self, Ty};
+use rustc_ast::ast;
 use rustc_errors::Applicability;
-use rustc_hir::*;
+use rustc_hir::{Expr, ExprKind, GenericArg, Mutability, QPath, TyKind, UnOp};
 use rustc_lint::{LateContext, LateLintPass};
+use rustc_middle::ty::{self, Ty};
 use rustc_session::{declare_lint_pass, declare_tool_lint};
 use std::borrow::Cow;
-use syntax::ast;
 
 declare_clippy_lint! {
     /// **What it does:** Checks for transmutes that can't ever be correct on any
@@ -28,6 +29,7 @@ declare_clippy_lint! {
     "transmutes that are confusing at best, undefined behaviour at worst and always useless"
 }
 
+// FIXME: Move this to `complexity` again, after #5343 is fixed
 declare_clippy_lint! {
     /// **What it does:** Checks for transmutes to the original type of the object
     /// and transmutes that could be a cast.
@@ -42,7 +44,7 @@ declare_clippy_lint! {
     /// core::intrinsics::transmute(t); // where the result type is the same as `t`'s
     /// ```
     pub USELESS_TRANSMUTE,
-    complexity,
+    nursery,
     "transmutes that have the same to and from types or could be a cast/coercion"
 }
 
@@ -315,7 +317,7 @@ impl<'a, 'tcx> LateLintPass<'a, 'tcx> for Transmute {
                         USELESS_TRANSMUTE,
                         e.span,
                         "transmute from a reference to a pointer",
-                        |db| {
+                        |diag| {
                             if let Some(arg) = sugg::Sugg::hir_opt(cx, &args[0]) {
                                 let rty_and_mut = ty::TypeAndMut {
                                     ty: rty,
@@ -328,7 +330,7 @@ impl<'a, 'tcx> LateLintPass<'a, 'tcx> for Transmute {
                                     arg.as_ty(cx.tcx.mk_ptr(rty_and_mut)).as_ty(to_ty)
                                 };
 
-                                db.span_suggestion(e.span, "try", sugg.to_string(), Applicability::Unspecified);
+                                diag.span_suggestion(e.span, "try", sugg.to_string(), Applicability::Unspecified);
                             }
                         },
                     ),
@@ -337,9 +339,9 @@ impl<'a, 'tcx> LateLintPass<'a, 'tcx> for Transmute {
                         USELESS_TRANSMUTE,
                         e.span,
                         "transmute from an integer to a pointer",
-                        |db| {
+                        |diag| {
                             if let Some(arg) = sugg::Sugg::hir_opt(cx, &args[0]) {
-                                db.span_suggestion(
+                                diag.span_suggestion(
                                     e.span,
                                     "try",
                                     arg.as_ty(&to_ty.to_string()).to_string(),
@@ -384,7 +386,7 @@ impl<'a, 'tcx> LateLintPass<'a, 'tcx> for Transmute {
                              (`{}`)",
                             from_ty, to_ty
                         ),
-                        |db| {
+                        |diag| {
                             let arg = sugg::Sugg::hir(cx, &args[0], "..");
                             let (deref, cast) = if mutbl == Mutability::Mut {
                                 ("&mut *", "*mut")
@@ -398,7 +400,7 @@ impl<'a, 'tcx> LateLintPass<'a, 'tcx> for Transmute {
                                 arg.as_ty(&format!("{} {}", cast, get_type_snippet(cx, qpath, to_ref_ty)))
                             };
 
-                            db.span_suggestion(
+                            diag.span_suggestion(
                                 e.span,
                                 "try",
                                 sugg::make_unop(deref, arg).to_string(),
@@ -412,14 +414,14 @@ impl<'a, 'tcx> LateLintPass<'a, 'tcx> for Transmute {
                             TRANSMUTE_INT_TO_CHAR,
                             e.span,
                             &format!("transmute from a `{}` to a `char`", from_ty),
-                            |db| {
+                            |diag| {
                                 let arg = sugg::Sugg::hir(cx, &args[0], "..");
                                 let arg = if let ty::Int(_) = from_ty.kind {
                                     arg.as_ty(ast::UintTy::U32.name_str())
                                 } else {
                                     arg
                                 };
-                                db.span_suggestion(
+                                diag.span_suggestion(
                                     e.span,
                                     "consider using",
                                     format!("std::char::from_u32({}).unwrap()", arg.to_string()),
@@ -440,24 +442,19 @@ impl<'a, 'tcx> LateLintPass<'a, 'tcx> for Transmute {
                                     ""
                                 };
 
-                                span_lint_and_then(
+                                span_lint_and_sugg(
                                     cx,
                                     TRANSMUTE_BYTES_TO_STR,
                                     e.span,
                                     &format!("transmute from a `{}` to a `{}`", from_ty, to_ty),
-                                    |db| {
-                                        db.span_suggestion(
-                                            e.span,
-                                            "consider using",
-                                            format!(
-                                                "std::str::from_utf8{}({}).unwrap()",
-                                                postfix,
-                                                snippet(cx, args[0].span, ".."),
-                                            ),
-                                            Applicability::Unspecified,
-                                        );
-                                    }
-                                )
+                                    "consider using",
+                                    format!(
+                                        "std::str::from_utf8{}({}).unwrap()",
+                                        postfix,
+                                        snippet(cx, args[0].span, ".."),
+                                    ),
+                                    Applicability::Unspecified,
+                                );
                             } else {
                                 if cx.tcx.erase_regions(&from_ty) != cx.tcx.erase_regions(&to_ty) {
                                     span_lint_and_then(
@@ -465,7 +462,7 @@ impl<'a, 'tcx> LateLintPass<'a, 'tcx> for Transmute {
                                         TRANSMUTE_PTR_TO_PTR,
                                         e.span,
                                         "transmute from a reference to a reference",
-                                        |db| if let Some(arg) = sugg::Sugg::hir_opt(cx, &args[0]) {
+                                        |diag| if let Some(arg) = sugg::Sugg::hir_opt(cx, &args[0]) {
                                             let ty_from_and_mut = ty::TypeAndMut {
                                                 ty: ty_from,
                                                 mutbl: from_mutbl
@@ -479,7 +476,7 @@ impl<'a, 'tcx> LateLintPass<'a, 'tcx> for Transmute {
                                             } else {
                                                 sugg_paren.addr_deref()
                                             };
-                                            db.span_suggestion(
+                                            diag.span_suggestion(
                                                 e.span,
                                                 "try",
                                                 sugg.to_string(),
@@ -496,10 +493,10 @@ impl<'a, 'tcx> LateLintPass<'a, 'tcx> for Transmute {
                         TRANSMUTE_PTR_TO_PTR,
                         e.span,
                         "transmute from a pointer to a pointer",
-                        |db| {
+                        |diag| {
                             if let Some(arg) = sugg::Sugg::hir_opt(cx, &args[0]) {
                                 let sugg = arg.as_ty(cx.tcx.mk_ptr(to_ty));
-                                db.span_suggestion(e.span, "try", sugg.to_string(), Applicability::Unspecified);
+                                diag.span_suggestion(e.span, "try", sugg.to_string(), Applicability::Unspecified);
                             }
                         },
                     ),
@@ -509,10 +506,10 @@ impl<'a, 'tcx> LateLintPass<'a, 'tcx> for Transmute {
                             TRANSMUTE_INT_TO_BOOL,
                             e.span,
                             &format!("transmute from a `{}` to a `bool`", from_ty),
-                            |db| {
+                            |diag| {
                                 let arg = sugg::Sugg::hir(cx, &args[0], "..");
                                 let zero = sugg::Sugg::NonParen(Cow::from("0"));
-                                db.span_suggestion(
+                                diag.span_suggestion(
                                     e.span,
                                     "consider using",
                                     sugg::make_binop(ast::BinOpKind::Ne, &arg, &zero).to_string(),
@@ -526,7 +523,7 @@ impl<'a, 'tcx> LateLintPass<'a, 'tcx> for Transmute {
                         TRANSMUTE_INT_TO_FLOAT,
                         e.span,
                         &format!("transmute from a `{}` to a `{}`", from_ty, to_ty),
-                        |db| {
+                        |diag| {
                             let arg = sugg::Sugg::hir(cx, &args[0], "..");
                             let arg = if let ty::Int(int_ty) = from_ty.kind {
                                 arg.as_ty(format!(
@@ -536,7 +533,7 @@ impl<'a, 'tcx> LateLintPass<'a, 'tcx> for Transmute {
                             } else {
                                 arg
                             };
-                            db.span_suggestion(
+                            diag.span_suggestion(
                                 e.span,
                                 "consider using",
                                 format!("{}::from_bits({})", to_ty, arg.to_string()),
@@ -549,7 +546,7 @@ impl<'a, 'tcx> LateLintPass<'a, 'tcx> for Transmute {
                         TRANSMUTE_FLOAT_TO_INT,
                         e.span,
                         &format!("transmute from a `{}` to a `{}`", from_ty, to_ty),
-                        |db| {
+                        |diag| {
                             let mut expr = &args[0];
                             let mut arg = sugg::Sugg::hir(cx, expr, "..");
 
@@ -580,7 +577,7 @@ impl<'a, 'tcx> LateLintPass<'a, 'tcx> for Transmute {
                                 arg
                             };
 
-                            db.span_suggestion(
+                            diag.span_suggestion(
                                 e.span,
                                 "consider using",
                                 arg.to_string(),
