@@ -1,5 +1,3 @@
-#![allow(unused_variables)]
-
 #[repr(u32)]
 enum Bool { True }
 
@@ -12,5 +10,5 @@ fn main() {
     let mut x = Bool::True;
     evil(&mut x);
     let _y = x; // reading this ought to be enough to trigger validation
-    //~^ ERROR invalid enum discriminant 44
+    //~^ ERROR encountered 0x0000002c, but expected a valid enum discriminant
 }
