@@ -20,6 +20,7 @@ use std::rc::Rc;
 use failure::Fail;
 use ignore;
 use rustc_ast::ast;
+use rustc_span::symbol;
 
 use crate::comment::LineClasses;
 use crate::emitter::Emitter;
@@ -524,7 +525,7 @@ impl Input {
                 let file_stem = file.file_stem()?;
                 if file.parent()?.to_path_buf().join(file_stem).is_dir() {
                     Some(DirectoryOwnership::Owned {
-                        relative: file_stem.to_str().map(ast::Ident::from_str),
+                        relative: file_stem.to_str().map(symbol::Ident::from_str),
                     })
                 } else {
                     None
